package com.emr.dao;

import com.emr.dto.Archive_MasterDto;
import com.emr.entity.Archive_Master;
import com.emr.entity.Archive_Master_Vo;
import com.emr.entity.CommomVo;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface Archive_MasterMapper  {
    /**
     * 根据主键删除记录
     * @param id
     * @return
     */
    int deleteByPrimaryKey(String id);

    /**
     * 插入记录
     * @param record
     * @return
     */
    int insert(Archive_Master record);

    /**
     * 根据可选字段插入记录
     * @param record
     * @return
     */
    int insertSelective(Archive_Master record);

    /**
     * 根据id查询记录
     * @param id
     * @return
     */
    Archive_Master selectById(String id);

    /**
     * 根据字段查询记录
     * @param archiveMaster
     * @return
     */
    List<Archive_Master> selByCol(Archive_Master archiveMaster);

    /**
     * 根据可选字段查询住院相关记录
     * @param record
     * @return
     */
    List<Archive_Master> selectByCol(Archive_Master_Vo record);


    /**
     * 根据条件查找在院病案列表
     * @param archiveMasterVo
     * @return
     */
    List<Archive_Master> selectByInpAndVisitId(Archive_Master_Vo archiveMasterVo);

    /**
     * 根据可选字段查询查询出院相关记录
     * @param record
     * @return
     */
    List<Archive_Master_Vo> selectByColumn(Archive_Master_Vo record);

    /**
     * 根据可选字段查询查询终审相关记录
     * @param record
     * @return
     */
    List<Archive_Master_Vo> selectByLast(Archive_Master_Vo record);


    /**
     * 根据可选字段查询查询病案示踪相关记录
     * @param record
     * @return
     */
    List<Archive_Master_Vo> selectByUnfile(Archive_Master_Vo record);

    /**
     * 根据可选字段查询查询病案示踪相关记录(不含婴儿即inpNo里有B字母的数据)
     * @param record
     * @return
     */
    List<Archive_Master_Vo> selectByUnfileNew(Archive_Master_Vo record);

    //List<Archive_Master> selectByUnfile2(Archive_Master_Vo record);
    /**
     * 导出超7天的，初审和已归档
     * @param record
     * @return
     */
     //List<Archive_Master> sel7DayByCol(Archive_Master_Vo record);

    /**
     * 根据可选字段查询查询主任审核相关记录
     * @param record
     * @return
     */
    List<Archive_Master_Vo> selectByCmtCol(Archive_Master_Vo record);

    /**
     *
     * @param record
     * @return
     */
    List<Archive_Master> bbSel(Archive_Master_Vo record);

    /**
     *
     * @param record
     * @return
     */
    List<Archive_Master> bbSel2(Archive_Master_Vo record);

    /**
     * 根据可选字段修改记录
     * @param record
     * @return
     */
    int updateByClo(Archive_Master record);

    //根据id删除已归档数据
    int deleteRecordByInp(@Param("masterId")String masterId);

    /**
     * 根据条件查询记录
     * @param record
     * @return
     */
    int updateByClo2(Archive_Master record);

    /**
     * 根据id修改记录
     * @param record
     * @return
     */
    int updateById(Archive_Master record);

    List<Archive_Master> selectCommonlyUsed(String id);

    int insertCommonlyUsed(CommomVo commomVo);

    /**
     *
     * @param record
     * @return
     */
    List<Archive_Master> getDoctorInCharge(Archive_Master record);

    /**
     * 查询病案室操作记录
     */
    List<Archive_Master_Vo> selectHandleDetail(Archive_Master_Vo record);

    /**
     * 查询初次提交归档-超期天数列表
     * @param record
     * @return
     */
    List<Archive_Master_Vo> selectOverdue(Archive_Master_Vo record);

    List<Archive_Master_Vo> selectOverdueE(Archive_Master_Vo record);
    //获取初次审核信息
    List<Archive_Master_Vo>getFollowingInfo(@Param("ids")String ids);

    /**
     * 查询住院返修病历-超期天数列表
     * @param record
     * @return
     */
    List<Archive_Master_Vo> selectOverdue2(Archive_Master_Vo record);

    /**
     * 查询初次未提交-超期天数列表
     * @param archiveMasterVo
     * @return
     */
    List<Archive_Master_Vo> selectOverdue3(Archive_Master_Vo archiveMasterVo);

    /**
     * 查询返修未提交-超期天数列表
     * @param archiveMasterVo
     * @return
     */
    List<Archive_Master_Vo> selectOverdue4(Archive_Master_Vo archiveMasterVo);

    Archive_Master_Vo againTime(String masterId);

//    Archive_Master_Vo againOverdueTime(String masterId);


    //封存日志
    List<Archive_Master_Vo> selectByFollowinglog(Archive_Master_Vo record);


    List<Archive_Master_Vo>selectFollowing(@Param(value = "ids") String ids);

    /**
     * 查询病案是否存在
     * */
    List<Archive_Master>  selectByObject(@Param(value = "patientId") String patientId,@Param(value = "visitId")String visitId);


    /**
     * 查询留观病历列表
     * @param master
     * @return
     */
    List<Archive_Master> getObservationRecordList(Archive_Master_Vo master);

    /**
     * 查询急诊留观病历报表
     * @param master
     * @return
     */
    List<Archive_Master_Vo>getObservationReportList(Archive_Master_Vo master);

    /**
     * 封存查询记录
     * @param record
     * @return
     */
    List<Archive_Master_Vo> selectStorageByCol(Archive_Master_Vo record);


    /**
     * 复印记录明细
     * @param record
     * @return
     */
    List<Archive_Master_Vo> copyDetailList(Archive_Master_Vo record);


    /**
     * 复印记录统计
     * @param record
     * @return
     */
    List<Archive_Master_Vo> copyCountList(Archive_Master_Vo record);



    /**
     * 查询记录数量
     * @param archiveMasterVo
     * @return
     */
    Integer selectCountNum(Archive_Master_Vo archiveMasterVo);

    /**
     * 查询标识
     * @param archiveMasterVo
     * @return
     */
    Integer selectTime1(Archive_Master_Vo archiveMasterVo);

    List<Archive_MasterDto> selectChildMasterId(String time);
    /**
     * @description: 查询医生缺陷数量
     * @params:  id
     * @return: Archive_Master_Vo
     * @author linjj
     * @date: 2023/6/29 17:17
     */
    List<Archive_Master_Vo> getDoctorDefectNum(@Param(value = "ids")String ids);

    /**
     * @description: 查询护士缺陷数量
     * @params:  id
     * @return: Archive_Master_Vo
     * @author linjj
     * @date: 2023/6/29 17:17
     */
    List<Archive_Master_Vo> getNurseDefectNum(@Param(value = "ids")String ids);
    /**
     * @description: 查询死亡病历
     * @params:  archiveMasterVo
     * @return: Archive_Master_Vo
     * @author linjj
     * @date: 2025/1/8 15:41
     */
    List<Archive_Master> getDeathRecordList(Archive_Master_Vo archiveMasterVo);
}